@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-// $Id: dnspacket.cc,v 1.24 2004/02/04 22:41:17 ahu Exp $
+// $Id: dnspacket.cc,v 1.32 2005/01/11 19:59:00 ahu Exp $
 #include "utility.hh"
 #include <cstdio>
 
@@ -107,7 +107,7 @@ int DNSPacket::expand(const unsigned char *begin, const unsigned char *end, stri
 
     if((n & 0xc0) == 0xc0 ) { 
        unsigned int labelOffset=(n&~0xc0)*256+ (int)*(unsigned char *)p;
-       expand((unsigned char *)stringbuffer.c_str()+labelOffset,end,expanded,depth++);
+       expand((unsigned char *)stringbuffer.c_str()+labelOffset,end,expanded,++depth); // was cvstrac ticket #21
        return 1+p-begin;
     }
 
@@ -231,7 +231,8 @@ void DNSPacket::addRecord(const DNSResourceRecord &rr)
   if(d_compress)
     for(vector<DNSResourceRecord>::const_iterator i=rrs.begin();i!=rrs.end();++i) 
       if(rr.qname==i->qname && rr.qtype==i->qtype && rr.content==i->content)
-	return;
+	if(rr.qtype.getCode()!=QType::MX || rr.priority==i->priority)
+	  return;
 
   rrs.push_back(rr);
 }
@@ -437,7 +438,7 @@ void DNSPacket::fillSOAData(const string &content, SOAData &data)
     data.hostmaster=attodot(parts[1]); // ahu@ds9a.nl -> ahu.ds9a.nl, piet.puk@ds9a.nl -> piet\.puk.ds9a.nl
 
   if(pleft>2)
-    data.serial=atoi(parts[2].c_str());
+    data.serial=strtoul(parts[2].c_str(), NULL, 10);
 
   if(pleft>3)
     data.refresh=atoi(parts[3].c_str());
@@ -1205,7 +1206,7 @@ vector<DNSResourceRecord> DNSPacket::getAnswers()
 
   int numanswers=ntohs(d.ancount) + ntohs(d.nscount) + ntohs(d.arcount);
   int length;
-  int pos=0;
+  u_int16_t pos=0;
   while(numanswers--) {
     string name;  
     int offset=0;
@@ -1271,10 +1272,13 @@ vector<DNSResourceRecord> DNSPacket::getAnswers()
       rr.content.assign((const char *)datapos+offset+1,(int)datapos[offset]);
       break;
 
-    case QType::HINFO:
-      rr.content.assign((const char *)datapos+offset+1,(int)datapos[offset]);
-      rr.content+=" ";
-      rr.content.append((const char *)datapos+offset+rr.content.size(),(int)datapos[offset+rr.content.size()-1]);
+    case QType::HINFO:  // this code is way way way overdue for a redesign
+      rr.content="\"";
+      rr.content.append((const char *)datapos+offset+1,(int)datapos[offset]);
+      rr.content+="\" \"";
+      rr.content.append((const char *)datapos+offset+(int)datapos[offset]+2,
+			(int)datapos[offset+(int)datapos[offset]+1]);
+      rr.content+="\"";
       break;
 
 
@@ -1283,7 +1287,7 @@ vector<DNSResourceRecord> DNSPacket::getAnswers()
       break;
 
 
-    case QType::SRV: // rfc 2025
+    case QType::SRV: // rfc 2052
       // priority goes into mx-priority
       rr.priority=(datapos[0] << 8) + datapos[1];
       // rest glue together  
@@ -1294,6 +1298,50 @@ vector<DNSResourceRecord> DNSPacket::getAnswers()
       rr.content+=" "+itoa(port)+" "+part;
       break;
 
+    case QType::NAPTR: // rfc 2915
+      {
+        const string quote="\"";
+        const string space=" ";
+
+        int order;
+        int pref;
+        string flags, services, regex, replacement, result;
+
+        order=(datapos[0] << 8) + datapos[1];
+
+        // "pref" should maybe be put into rr.priority, but that
+        // might have unintended side effects that I cannot
+        // evaluate at this time.
+        //
+        // Lorens Kockum 2004-10-12
+
+        pref=(datapos[2] << 8) + datapos[3];
+
+        // The following would be a good subject for a mini-
+        // function with boundary checking, which could be
+        // reused in a lot of places in this file (see FIXME
+        // at beginning of function).
+
+        offset = 4 ;
+        flags.assign((const char *)datapos+offset+1,(int)datapos[offset]);
+        offset+=flags.size()+1;
+        services.assign((const char *)datapos+offset+1,(int)datapos[offset]);
+        offset+=services.size()+1;
+        regex.assign((const char *)datapos+offset+1,(int)datapos[offset]);
+        offset+=regex.size()+1;
+
+        expand(datapos+offset,end,replacement);
+
+        if (!replacement.size()) replacement = "." ;
+
+        rr.content = itoa(order) \
+                     + " " + itoa(pref) \
+                     + " " + quote + flags + quote \
+                     + " " + quote + services + quote \
+                     + " " + quote + regex + quote \
+                     + " " + replacement;
+      }
+      break;
 
     case QType::RP:
       offset+=expand(datapos+offset,end,rr.content);
@@ -1319,6 +1367,7 @@ vector<DNSResourceRecord> DNSPacket::getAnswers()
 
       rr.content=tmp;
       break;
+
     default:
       rr.qtype=rr.qtype.getCode()+1024;
       rr.content.assign((const char *)datapos,length);
