@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2002  PowerDNS.COM BV
+    Copyright (C) 2004  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -39,11 +39,16 @@
 #include <fstream>
 #include "recursor_cache.hh"
 
+#ifdef FreeBSD           // see cvstrac ticket #26
+#include <pthread.h>
+#include <semaphore.h>
+#endif
+
 MemRecursorCache RC;
 
 string s_programname="pdns_recursor";
 
-#ifndef WIN32
+#if !WIN32 && !FreeBSD
 extern "C" {
   int sem_init(sem_t*, int, unsigned int){return 0;}
   int sem_wait(sem_t*){return 0;}
@@ -64,9 +69,9 @@ ArgvMap &arg()
   static ArgvMap theArg;
   return theArg;
 }
-int d_clientsock;
-int d_serversock;
-int d_tcpserversock;
+static int d_clientsock;
+static int d_serversock;
+static int d_tcpserversock;
 
 struct PacketID
 {
@@ -238,6 +243,8 @@ void makeClientSocket()
   }
   if(!tries)
     throw AhuException("Resolver binding to local socket: "+stringerror());
+
+  Utility::setNonBlocking(d_clientsock);
 }
 
 void makeTCPServerSocket()
@@ -260,15 +267,18 @@ void makeTCPServerSocket()
   }
 
   sin.sin_port = htons(arg().asNum("local-port")); 
-    
-  if (bind(d_tcpserversock, (struct sockaddr *)&sin, sizeof(sin))<0) 
-    throw AhuException("TCP Resolver binding to server socket: "+stringerror());
-  
+
   int tmp=1;
   if(setsockopt(d_tcpserversock,SOL_SOCKET,SO_REUSEADDR,(char*)&tmp,sizeof tmp)<0) {
     L<<Logger::Error<<"Setsockopt failed"<<endl;
     exit(1);  
   }
+    
+  if (bind(d_tcpserversock, (struct sockaddr *)&sin, sizeof(sin))<0) 
+    throw AhuException("TCP Resolver binding to server socket: "+stringerror());
+  
+
+  Utility::setNonBlocking(d_tcpserversock);
 
   listen(d_tcpserversock, 128);
 }
@@ -299,6 +309,9 @@ void makeServerSocket()
     
   if (bind(d_serversock, (struct sockaddr *)&sin, sizeof(sin))<0) 
     throw AhuException("Resolver binding to server socket: "+stringerror());
+
+  Utility::setNonBlocking(d_serversock);
+
   L<<Logger::Error<<"Incoming query source port: "<<arg().asNum("local-port")<<endl;
 }
 
@@ -393,7 +406,7 @@ int main(int argc, char **argv)
     arg().set("soa-serial-offset","Don't change")="0";
     arg().set("aaaa-additional-processing","turn on to do AAAA additional processing (slow)")="off";
     arg().set("local-port","port to listen on")="53";
-    arg().set("local-address","port to listen on")="0.0.0.0";
+    arg().set("local-address","single address to listen on")="0.0.0.0";
     arg().set("trace","if we should output heaps of logging")="off";
     arg().set("daemon","Operate as a daemon")="yes";
     arg().set("quiet","Suppress logging of questions and answers")="off";
@@ -476,7 +489,6 @@ int main(int argc, char **argv)
 	fdmax=max(fdmax,i->fd);
       }
 
-
       /* this should listen on a TCP port as well for new connections,  */
       int selret = select(  fdmax + 1, &readfds, NULL, NULL, &tv );
       if(selret<=0) 
@@ -532,9 +544,9 @@ int main(int argc, char **argv)
 	struct sockaddr_in addr;
 	socklen_t addrlen=sizeof(addr);
 	int newsock=accept(d_tcpserversock, (struct sockaddr*)&addr, &addrlen);
-	Utility::setNonBlocking(newsock);
-	
+
 	if(newsock>0) {
+	  Utility::setNonBlocking(newsock);
 	  TCPConnection tc;
 	  tc.fd=newsock;
 	  tc.state=TCPConnection::BYTE0;
