@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-// $Id: receiver.cc,v 1.11 2004/01/16 22:18:12 ahu Exp $
+// $Id: receiver.cc,v 1.13 2005/01/11 19:24:27 ahu Exp $
 #include <cstdio>
 #include <signal.h>
 #include <cstring>
@@ -97,11 +97,15 @@ void daemonize(void)
   
   setsid(); 
 
-  // cleanup open fds, but skip sockets 
-  close(0);
-  close(1);
-  close(2);
-
+  int i=open("/dev/null",O_RDWR); /* open stdin */
+  if(i < 0) 
+    L<<Logger::Critical<<"Unable to open /dev/null: "<<stringerror()<<endl;
+  else {
+    dup2(i,0); /* stdin */
+    dup2(i,1); /* stderr */
+    dup2(i,2); /* stderr */
+    close(i);
+  }
 }
 
 
@@ -550,7 +554,7 @@ int main(int argc, char **argv)
   declareStats();
   DLOG(L<<Logger::Warning<<"Verbose logging in effect"<<endl);
   
-  L<<Logger::Warning<<"PowerDNS "<<VERSION<<" (C) 2001-2004 PowerDNS.COM BV ("<<__DATE__", "__TIME__<<") starting up"<<endl;
+  L<<Logger::Warning<<"PowerDNS "<<VERSION<<" (C) 2001-2005 PowerDNS.COM BV ("<<__DATE__", "__TIME__<<") starting up"<<endl;
 
   L<<Logger::Warning<<"PowerDNS comes with ABSOLUTELY NO WARRANTY. "
     "This is free software, and you are welcome to redistribute it "
